/***************************************************************
 HomeAuto.c

 X10 Home Automation Application for Imagine Tools Ethernet
 Starter Kit.

 Set IP Address in TCP_CONFIG.LB.

 The macro X10_DEFINEDEV may be used to define additional
 devices at compile time. Devices can be added, edited,
 deleted at run-time through the web browser.

 Instructions for run-time configuration are contained in
 the web pages.

 Set browser user name and password below
***************************************************************/

#define USERNAME "it"
#define PASSWORD "it"

#use    X10.LIB
#use    hobbyist.lib

#define TCPCONFIG 1
#define USE_RABBITWEB 1

// Make sure the product of these two macros is a multiple of 8
//  so we don't lose bytes in the integer division below
#define MAX_X10DEVICES 16
#define MAX_COMMANDS_PD_PD 48  // commands per day per device

#define X10_COMMNDBITS 4
#define X10_SCHEDBYTES ((MAX_COMMANDS_PD_PD*7*X10_COMMNDBITS)/8)
#define MAX_X10TEXTLEN 32

// This macro is for defining devices at compile time to be
// initialized on program start up. Devices may be defined
// at run time also. Indx should be 1-16 and unique. Title
// should be a string in quotes or a pointer to a NULL-
// terminated string.
#define X10_DEFINEDEV(indx,hc,kc,en,title) \
  	x10device[indx-1].housecode = hc;       \
	x10device[indx-1].keycode = kc;         \
  	x10device[indx-1].enabled = en;         \
  	x10device[indx-1].used = 1;             \
   memcpy(x10device[indx-1].text,title,strlen(title));

#use "dcrtcp.lib"
#use "http.lib"

#ximport "x10configpage.zhtml" x10configPage
#ximport "x10_main.zhtml"  x10main
#ximport "ITBanner.gif" ITBanner
#ximport "chipbkgrnd.gif" chipbkgrnd

int getX10command(int device, int wdTemp, int hTemp, int mTemp);

/* The default mime type for '/' must be first */
SSPEC_MIMETABLE_START
   // This handler enables the ZHTML parser to be used on ZHTML files...
   SSPEC_MIME_FUNC(".zhtml", "text/html", zhtml_handler),
   SSPEC_MIME(".html", "text/html"),
   SSPEC_MIME(".gif", "image/gif")
SSPEC_MIMETABLE_END

/* Associate the #ximported files with the web server */
SSPEC_RESOURCETABLE_START
   SSPEC_RESOURCE_XMEMFILE("/index.zhtml", x10main),
   SSPEC_RESOURCE_XMEMFILE("/admin/index.zhtml", x10configPage)
SSPEC_RESOURCETABLE_END

struct x10_Device{
   int  used;
   int  enabled;
   int  housecode;
   int  keycode;
   char text[MAX_X10TEXTLEN];
   char schedule[X10_SCHEDBYTES];
	int  togglebutton;
	char lastSWposition;
};

struct x10_Device x10device[MAX_X10DEVICES];

#web_groups admin

int appMessage;
#web appMessage

int buttonCode;
#web buttonCode

int unitMonth;
#web unitMonth
int unitDay;
#web unitDay (($unitDay >= 0) && ($unitDay <= 31))
int unitHour;
#web unitHour (($unitHour >= 0) && ($unitHour <= 23))
int unitMinute;
#web unitMinute (($unitMinute >= 0) && ($unitMinute <= 60))
int unitSecond;
#web unitSecond (($unitSecond >= 0) && ($unitSecond <= 60))
int x10year;
#web x10year ( ($x10year > 1979) && ($x10year < 2116) )

int x10Day;
#web x10Day
int x10enabled;
#web x10enabled
int editHC;
#web editHC
int editKC;
#web editKC
int editIndex;
#web editIndex

int 	timeradio00[24];
int 	timeradio30[24];
#web  timeradio30
#web  timeradio00
#web  timeradio00[@] select("0" = 0, "1" = 1, "2" = 2)
#web  timeradio30[@] select("0" = 0, "1" = 1, "2" = 2)

#web  x10device[@]
#web  x10device[@].togglebutton select("0" = 0, "1" = 1)
#web  x10device[@].enabled select("0" = 0, "1" = 1)

#web  editHC select("A"=0,"B","C","D","E","F","G","H", \
					     "I","J","K","L","M","N","O","P")

#web  editKC select( "1"=0,"2","3","4","5","6","7","8", \
        					"9","10","11","12","13","14","15","16")

#web  editIndex select( "DEV_01"=0,"DEV_02","DEV_03","DEV_04","DEV_05",   \
		 	"DEV_06","DEV_07","DEV_08", "DEV_09","DEV_10","DEV_11", "DEV_12",\
         "DEV_13","DEV_14","DEV_15","DEV_16")

#web  unitMonth select("January"=0,"February","March","April","May",  	\
				  	"June","July","August","September","October","November", \
              	"December")

#web 	x10Day select("All"=0, "Monday", "Tuesday", "Wednesday", \
 						 "Thursday","Friday", "Saturday", "Sunday" )
int currentDev;

void main(void)
{
   int userid, wdTemp, hTemp, mTemp;
   int i, command, offset;

	HBx10_Init();

   // Zero out entire array of device structures
   memset(x10device,0,MAX_X10DEVICES*sizeof(struct x10_Device));

	// Define default devices
   X10_DEFINEDEV(1,0,0,1,"Device A-0");

   memset(timeradio00,0,48);
   memset(timeradio30,0,48);

   // Force copy of device 1 info into temp web variables
   currentDev = 1;

   appMessage = 0;
   editIndex  = 0;
   buttonCode = 0;
   appMessage = 0;
   x10Day = 0;

   sock_init();
   http_init();
   http_set_path("/", "index.zhtml");
   tcp_reserveport(80);

   sspec_addrule("/", "Admin", admin, admin, SERVER_HTTP,
                 SERVER_AUTH_BASIC, NULL);
   sspec_addxmemfile("ITBanner.gif", ITBanner, SERVER_HTTP);
   sspec_addxmemfile("chipbkgrnd.gif", chipbkgrnd, SERVER_HTTP);

   // Username = password = it
   userid = sauth_adduser(USERNAME, PASSWORD, SERVER_HTTP);
   sauth_setusermask(userid, admin, NULL);

	x10year    = HBrtcGetYear();
   unitMonth  = HBrtcGetMonth();
	unitDay    = HBrtcGetDay();
	unitHour   = HBrtcGetHour();
	unitMinute = HBrtcGetMinute();
	unitSecond = HBrtcGetSecond();

	while(1)
   {
      // This drives the HTTP server.
      http_handler();
   	if(currentDev!=editIndex)
      {
			readSchedule(editIndex,0);
			currentDev = editIndex;
			editHC =  x10device[currentDev].housecode;
			editKC =  x10device[currentDev].keycode;
      }

		// The switch statement handles mesages sent from the
      //  browser interface in "hidden" inputs on HTML forms.
		if(appMessage) {
      	switch(appMessage)
	      {
   			case 1 :    // Copy configuration to selected device
	            x10device[editIndex].housecode = editHC;
   	         x10device[editIndex].keycode = editKC;
   	         x10device[editIndex].used = 1;
   	      	break;
   			case 2 :    // Prepare schedule data
					printf("preparing device schedule data\n");
					readSchedule(editIndex,x10Day);
   	      	break;
   			case 3 :    // Process new schedule info
					writeSchedule(editIndex,x10Day);
					break;
   			case  4:		// Delete device
               memset(&(x10device[editIndex].used),0,
               			sizeof(struct x10_Device)) ;
   	      	break;
            case 5:    // Refresh time display
					x10year    = HBrtcGetYear();
               unitMonth  = HBrtcGetMonth();
					unitDay    = HBrtcGetDay();
					unitHour   = HBrtcGetHour();
					unitMinute = HBrtcGetMinute();
					unitSecond = HBrtcGetSecond();
            	break;
            case 6:		// Set RCM3700 time
					HBrtcSetDate(unitDay, unitMonth, x10year);
					HBrtcSetTime(unitHour, unitMinute, unitSecond);
            	break;
            default:
            	break;
	      }
  	      appMessage = 0; // Reset message
   	}

      // This drives the HTTP server.
      http_handler();

      //********************************************
      // This task handles sending prescheduled commands
		costate
   	{
		  // Wait for 30 minutes time slots
			waitfor(DelaySec(55));
        	if( (HBrtcGetMinute() % 30) == 0) {

	         wdTemp = HBrtcGetWeekday();
   	      hTemp  = HBrtcGetHour();
      	   mTemp  = HBrtcGetMinute();

	 			for(i=0; i<MAX_X10DEVICES; i++) {
					if(x10device[i].used && x10device[i].enabled)
  	      	   {
   	      	   command = getX10command(i,wdTemp, hTemp, mTemp);
						if(command==1) {
							   waitfor(HBx10_SendCommand(x10device[i].housecode+'A',
	  											x10device[i].keycode,
   	  	      	      	         X10_OFF)==0);
	               }
  						else if(command==2) {
							   waitfor(HBx10_SendCommand(x10device[i].housecode+'A',
	  											x10device[i].keycode,
   	  	      		               X10_ON)==0);
	               }
					}	// End if dev used
				}	// End for
			}	// End if
      }	// End costate

      //********************************************
      // This task handles toggling devices from the
      // browser interface's front page
		costate
   	{
			// Loop through all devices
			for(i=0; i<MAX_X10DEVICES; i++)
	      {
            // If device used and switch changed
				if(x10device[i].used &&
            	x10device[i].lastSWposition != x10device[i].togglebutton
               )
  	      	{
				   // Reset toggle flag
			     	x10device[i].lastSWposition = x10device[i].togglebutton;
  	   			if(!x10device[i].lastSWposition)  // switch off if on
   	         {
               	// Initiate off command when command pipeline clear.
                  // Once initiated, the transimission is driven by
                  // An external interrupt and a timer interrupt.
                  // X10 messages take nearly a second to send, so
                  // don't want to busy wait!
					   waitfor(HBx10_SendCommand(x10device[i].housecode+'A',
           	  									x10device[i].keycode,
              	  	                     X10_OFF)==0);
	   			}
		      	else // switch on if off
  		      	{
               	// Initiate on command when command pipeline clear
					   waitfor(HBx10_SendCommand(x10device[i].housecode+'A',
     	   	   								x10device[i].keycode,
       	   	                        X10_ON)==0);
	      		} 	// End else
     			}	// End if
			}	// End for loop
		}	// End toggling task
	}   // End outer while loop
}

//***** Pack schedule as 4 bit commands
// (Could use 2 bits, but 4 allows for future command set enhancement)
// A more efficient method would be to keep a time-sorted linked list
// of command events, but the programming would be trickier.
nodebug
writeSchedule(int device, int x10Day){
   int i,j;
   char *ptr;

   if(x10Day){  // if just one day
		// initialize ptr to the right spot in this device's schedule array
	   ptr = (char*)((unsigned)x10device[device].schedule +
  				(x10Day-1)*MAX_COMMANDS_PD_PD*X10_COMMNDBITS);

	   for(i=0; i<24; i++)
   	{
			// Even bytes in the integer arrays will hold the command values
         //  in the lowest nibble (1 nibble = 4 bits = 1/2 byte)
	   	*ptr = (char)timeradio00[i]  +       // hour command in low nibble
					 (char)(timeradio30[i] << 4  ) ; // half hour in high nibble
      	 ptr++;    // advance pointer
	   }
   }
   else  // All 7 days of the week
   {
   	for(i=0; i<7 ;i++){
			// Initialize to ith day of the week
			ptr = (char*)((unsigned)x10device[device].schedule +
  					i*MAX_COMMANDS_PD_PD*X10_COMMNDBITS/8);
			for(j=0; j<24; j++)
			{
		   	*ptr = (char)timeradio00[j]  +
					 (char)(timeradio30[j] << 4  ) ;
   	   	ptr++;
			}
      }
   }
}

//***** Unpack schedule into timeradio int arrays to
//*****  to use in HTML interface
nodebug
readSchedule(int device, int x10Day){
   int i;
   char *ptr;

   if(x10Day){  // if just one day
	   ptr = (char*)((unsigned)x10device[device].schedule +
  			(x10Day-1)*MAX_COMMANDS_PD_PD*X10_COMMNDBITS/8);
   }
   else {
	   ptr = (char*)((unsigned)x10device[device].schedule);
   }
   for(i=0; i<24; i++)
  	{
		timeradio00[i] = *ptr & 0x0F;
		timeradio30[i] = (*ptr & 0xF0) >> 4;
		ptr++;
   }
}

nodebug
int getX10command(int device, int wdTemp, int hTemp, int mTemp)
{
	int  offset, odd;
   char command;

  	offset = wdTemp     * 48  + // 48 slots per day
        		hTemp      * 2   + // 2 slots per hour
	     		mTemp      / 30  ; // 1/30 of a slot per minute

	odd = 1 & offset;
	command = x10device[device].schedule[offset/2];

	// Because bytes are packed with 2 commands,
   //  need to know if it's odd or even to unpack it
 	if(odd)
 	{
 		command = (command & 0xF0)>>4;  // Use high nibble
 	}
 	else
   {
 	 	command = command & 0x0F;       // Use low nibble
 	}
  	return command;
}



